"""Base class for Google Calendar tools."""
from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Optional

import pytz
from langchain.pydantic_v1 import Field
from langchain.tools.base import BaseTool

from langchain_google_calendar_tools.utils import build_resource_service

if TYPE_CHECKING:
    # This is for linting and IDE typehints
    from googleapiclient.discovery import Resource  # type: ignore
else:
    try:
        # We do this so pydantic can resolve the types when instantiating
        from googleapiclient.discovery import Resource
    except ImportError:
        pass


class CalendarBaseTool(BaseTool):
    """Base class for Google Calendar tools."""

    name: str = "calendar_base_tool"
    description: str = "Base class for Google Calendar tools."
    api_resource: Resource = Field(default_factory=build_resource_service)

    def format_date(self, date_data: datetime) -> str:
        """Format date to RFC3339 format."""
        return date_data.isoformat()

    @property
    def timezone(self):
        # TODO: get timezone from user's calendar
        return pytz.timezone("Asia/Ho_Chi_Minh")

    def add_timezone(self, date: datetime) -> datetime:
        if date.tzinfo:
            return date

        return self.timezone.localize(date, is_dst=None)

    @classmethod
    def from_api_resource(cls, api_resource: Resource) -> "CalendarBaseTool":
        """Create a tool from an api resource.

        Args:
            api_resource: The api resource to use.

        Returns:
            A tool.
        """
        return cls(api_resource=api_resource)



from datetime import datetime
from typing import List, Optional

from langchain.pydantic_v1 import BaseModel, EmailStr, Field


class UpdateExistEventArgsSchema(BaseModel):
    """Input for UpdateExistEvent."""

    event_id: str = Field(
        ...,
        description="The id of the event to update. You can get it from list_events tool. It must not be empty.",
    )

    summary: str = Field(
        ...,
        description="The summary of the event.",
    )

    start_time: datetime = Field(
        ...,
        description="The start time of the event.",
    )

    end_time: datetime = Field(
        ...,
        description="The end time of the event.",
    )

    location: Optional[str] = Field(
        default=None,
        description="The location of the event.",
    )

    description: Optional[str] = Field(
        default=None,
        description="The description of the event.",
    )

    attendees: List[EmailStr] = Field(
        default=[],
        description="The attendees of the event.",
    )



from datetime import datetime
from typing import Optional

from langchain.pydantic_v1 import BaseModel, Field


class SearchArgsSchema(BaseModel):
    """Input for GetMessageTool."""

    num_events: int = Field(
        ...,
        description="The number of events to return.",
    )

    end_date: Optional[datetime] = Field(
        default=None,
        description="The maximum end date of the events to return.",
    )
    start_date: Optional[datetime] = Field(
        default=None,
        description="The minimum start date of the events to return.",
    )

    summary: Optional[str] = Field(
        default=None,
        description="The summary of the events to return, use for filtering.",
    )



from datetime import datetime
from typing import Dict, List, Optional, Type

from googleapiclient.errors import HttpError  # type: ignore
from langchain.callbacks.manager import CallbackManagerForToolRun
from langchain.pydantic_v1 import EmailStr

from langchain_google_calendar_tools.tools.base import CalendarBaseTool
from langchain_google_calendar_tools.tools.create_new_event.input_arg import (
    CreateEventArgsSchema,
    RecurringConfig,
)


class CreateNewEvent(CalendarBaseTool):
    """Tool that create new event in the user's calendar."""

    name: str = "create_new_event"
    description: str = (
        "Use this tool to create new event (not recurring events)"
        + " User must provide summary, start and end time."
        + " In addition, user can provide location, description, attendees."
        + " Reminder is not supported yet."
        + " Returns the event summary"
    )
    args_schema: Type[CreateEventArgsSchema] = CreateEventArgsSchema

    def __format_recurring_config(self, recurring_config: RecurringConfig) -> str:
        """Format recurring config to RFC3339 format."""

        rules = [
            f"FREQ={recurring_config.frequency}",
        ]

        if recurring_config.interval:
            rules.append(f"INTERVAL={recurring_config.interval}")

        if recurring_config.until is not None and recurring_config.count is not None:
            raise ValueError("UNTIL and COUNT cannot be used together")

        if recurring_config.count:
            rules.append(f"COUNT={recurring_config.count}")

        if recurring_config.until:
            rules.append(f"UNTIL={self.format_date(recurring_config.until)}")

        if recurring_config.byday:
            rules.append(f"BYDAY={','.join(recurring_config.byday)}")

        return 'RRULE:' + ';'.join(rules)

    def _run(
        self,
        summary: str,
        start_time: datetime,
        end_time: datetime,
        location: Optional[str] = None,
        description: Optional[str] = None,
        attendees: Optional[List[EmailStr]] = None,
        recurring_config: Optional[RecurringConfig] = None,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> Dict:
        """Run the tool."""

        start_time = self.add_timezone(start_time)
        end_time = self.add_timezone(end_time)

        event = {
            "summary": summary,
            "start": {
                "dateTime": self.format_date(start_time),
                "timeZone": start_time.tzinfo.zone,  # type: ignore
            },
            "end": {
                "dateTime": self.format_date(end_time),
                "timeZone": end_time.tzinfo.zone,  # type: ignore
            },
        }

        if location:
            event["location"] = location

        if description:
            event["description"] = description

        if attendees:
            event["attendees"] = [{"email": attendee} for attendee in attendees]  # type: ignore

        if recurring_config:
            try:
                event["recurrence"] = self.__format_recurring_config(recurring_config)
            except ValueError as e:
                return {"error": str(e)}

        try:
            response = self.api_resource.events().insert(calendarId="primary", body=event).execute()
        except HttpError as e:
            return {"error": {"code": e.resp.status, "message": e.reason}}

        return response



from datetime import datetime
from typing import Dict, List, Optional, Type

from googleapiclient.errors import HttpError  # type: ignore
from langchain.callbacks.manager import CallbackManagerForToolRun
from langchain.pydantic_v1 import EmailStr

from langchain_google_calendar_tools.tools.base import CalendarBaseTool
from langchain_google_calendar_tools.tools.update_exist_event.input_arg import (
    UpdateExistEventArgsSchema,
)


class UpdateExistEvent(CalendarBaseTool):
    """Tool that update exist event in the user's calendar."""

    name: str = "update_exist_event"
    # flake8: noqa
    description: str = (
        "Use this tool to update exist event"
        + " You must provide all 4 fields including: event_id, summary, end date and start date to update the event, it can be get from list_events tool. "
        + " In case summary, end date and start date need to be updated, use new data."
        + " You can update summary, start time, end time, location, description, attendees."
        + " Updating recurring event is not supported."
    )
    args_schema: Type[UpdateExistEventArgsSchema] = UpdateExistEventArgsSchema

    def _run(
        self,
        event_id: str,
        summary: str,
        start_time: datetime,
        end_time: datetime,
        location: Optional[str] = None,
        description: Optional[str] = None,
        attendees: Optional[List[EmailStr]] = None,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> Dict:
        """Run the tool."""

        if not event_id:
            return {"error": "event_id must not be empty. You can get it from list_events tool."}

        start_time = self.add_timezone(start_time)

        end_time = self.add_timezone(end_time)

        event = {
            "summary": summary,
            "start": {
                "dateTime": self.format_date(start_time),
                "timeZone": start_time.tzinfo.zone,  # type: ignore
            },
            "end": {
                "dateTime": self.format_date(end_time),
                "timeZone": end_time.tzinfo.zone,  # type: ignore
            },
        }

        if location:
            event["location"] = location

        if description:
            event["description"] = description

        if attendees:
            event["attendees"] = [{"email": str(attendee)} for attendee in attendees]  # type: ignore

        try:
            response = self.api_resource.events().update(eventId=event_id, calendarId="primary", body=event).execute()
        except HttpError as e:
            return {"error": {"code": e.resp.status, "message": e.reason}}

        return response



"""Base class for Google Calendar tools."""
from __future__ import annotations

from datetime import datetime
from typing import Optional

from langchain.callbacks.manager import CallbackManagerForToolRun
from langchain.tools.base import BaseTool


class GetCurrentDatetime(BaseTool):
    """Helper tool that get current datetime."""

    name: str = "get_current_datetime"
    description: str = "Use this tool to get current datetime."

    def _run(
        self,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> str:
        return datetime.now().isoformat() + 'Z'



from datetime import datetime
from typing import Dict, List, Literal, Optional

from langchain.pydantic_v1 import BaseModel, EmailStr, Field


class RecurringConfig(BaseModel):
    # flake8: noqa
    frequency: Literal["SECONDLY", "MINUTELY", "HOURLY", "DAILY", "WEEKLY", "MONTHLY", "YEARLY"] = Field(
        ...,
        description="The frequency of the recurring event, must be one of the following: SECONDLY, MINUTELY, HOURLY, DAILY, WEEKLY, MONTHLY, YEARLY",
    )

    interval: int = Field(
        ...,
        description="The interval between each event in the recurring event series, must be a positive integer, default is 1",
    )

    count: Optional[int] = Field(
        default=None,
        description="The number of times the event should recur. Optional.",
    )

    until: Optional[datetime] = Field(
        default=None,
        description="The date and time when the event should stop recurring. Optional. You can use either COUNT or UNTIL to specify the end of the event recurrence. Don't use both in the same rule.",
    )

    byday: Optional[Literal["SU", "MO", "TU", "WE", "TH", "FR", "SA"]] = Field(
        default=None,
        description="The days of the week that the event should recur. Optional. Must be one or more of the following: SU, MO, TU, WE, TH, FR, SA",
    )


class CreateEventArgsSchema(BaseModel):
    """Input for CreateNewEvent."""

    summary: str = Field(
        ...,
        description="The summary of the event.",
    )

    start_time: datetime = Field(
        ...,
        description="The start time of the event.",
    )

    end_time: datetime = Field(
        ...,
        description="The end time of the event.",
    )

    location: Optional[str] = Field(
        default=None,
        description="The location of the event.",
    )

    description: Optional[str] = Field(
        default=None,
        description="The description of the event.",
    )

    attendees: List[EmailStr] = Field(
        default=[],
        description="The attendees of the event.",
    )

    recurring_config: Optional[RecurringConfig] = Field(
        default=None,
        description="The recurring config of the event.",
    )



from datetime import datetime, timedelta
from typing import Dict, Optional, Type

from googleapiclient.errors import HttpError  # type: ignore
from langchain.callbacks.manager import CallbackManagerForToolRun

from langchain_google_calendar_tools.tools.base import CalendarBaseTool
from langchain_google_calendar_tools.tools.list_events.input_arg import SearchArgsSchema


class ListEvents(CalendarBaseTool):
    """Tool that gets k events from the user's calendar. You can filter by start date, end date and event summary."""

    name: str = "list_events"
    description: str = (
        "Use this tool to get k events from the user's calendar. "
        + " You can filter by start date, end date and event summary."
        + " Returns all events and their details."
    )

    args_schema: Type[SearchArgsSchema] = SearchArgsSchema

    def _run(
        self,
        num_events: int,
        summary: Optional[str] = None,
        run_manager: Optional[CallbackManagerForToolRun] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
    ) -> Dict:
        """Run the tool."""

        summary = summary.strip() if summary is not None else None

        if start_date is not None and start_date == end_date:
            end_date = start_date + timedelta(days=1)

        if start_date is not None:
            start_date = self.add_timezone(start_date)
        else:
            start_date = self.add_timezone(datetime.now())

        start_text = self.format_date(start_date)

        if end_date is not None:
            end_date = self.add_timezone(end_date)
            end_text = self.format_date(end_date)

        query = self.api_resource.events().list(
            calendarId="primary",
            timeMin=start_text,
            timeMax=end_text,
            maxResults=num_events,
            singleEvents=True,
            orderBy="startTime",
            q=summary,
        )

        try:
            response = query.execute()
        except HttpError as err:
            return {"error": {"code": err.resp.status, "message": err.reason}}

        return {
            "events": [
                {
                    "event_id": event.get("id"),
                    "summary": event.get("summary"),
                    "creator": event.get("creator", {}).get("email"),
                    "organizer": event.get("organizer", {}).get("email"),
                    "htmlLink": event.get("htmlLink"),
                    "start": {
                        "date": event.get("start", {}).get("date"),
                        "dateTime": event.get("start", {}).get("dateTime"),
                        "timeZone": event.get("start", {}).get("timeZone"),
                    },
                    "end": {
                        "date": event.get("start", {}).get("date"),
                        "dateTime": event.get("start", {}).get("dateTime"),
                        "timeZone": event.get("start", {}).get("timeZone"),
                    },
                    "location": event.get("location"),
                    "attendees": event.get("attendees", []),
                }
                for event in response.get("items", [])
                if summary is None or summary.lower() == event.get("summary", "").lower()
            ]
        }
